package com.chapter16.tankgame;

import javax.swing.*;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Vector;

//坦克大战的绘图区域 继承 JPanel 实现 KeyListener接口
public class MyPanel extends JPanel implements KeyListener, Runnable {

    // 定义我的坦克
    Hero hero = null;

    //定义敌方坦克，放入到Vector 集合中
    Vector<EnemyTank> enemyTanks = new Vector<>();
    // 坦克数量
    int enemyTankSize = 5;

    // 定义一个Vector ， 用于存放炸弹
    // 当子弹击中坦克时，加入bombs
    Vector<Bomb> bombs = new Vector<>();

    //定义三张炸弹图片，用于显示爆炸效果
    Image image1 = null;
    Image image2 = null;
    Image image3 = null;

    public MyPanel() {
        hero = new Hero(500, 600); // 初始化坦克
        hero.setSpeed(2);  // 初始化坦克速度

        // 初始化敌人的坦克
        for (int i = 0; i < enemyTankSize; i++) {
            // 创建一个敌人坦克的对象
            EnemyTank enemyTank = new EnemyTank(100 * (i + 1), 0);

            // 设置坦克的方向
            enemyTank.setDirect(1);

            //启动敌人坦克线程
            new Thread(enemyTank).start();

            // 给敌人坦克加入一个子弹
            Shot shot = new Shot(enemyTank.getX(), enemyTank.getY(), enemyTank.getDirect());
            enemyTank.shots.add(shot);
            // 启动 shot 对象
            new Thread(shot).start();

            // 将坦克对象，添加至列表中
            enemyTanks.add(enemyTank);

        }

        //初始化图片对象
        image1 = Toolkit.getDefaultToolkit().getImage(MyPanel.class.getResource("/bomb_1.png"));
        image2 = Toolkit.getDefaultToolkit().getImage(MyPanel.class.getResource("/bomb_2.png"));
        image3 = Toolkit.getDefaultToolkit().getImage(MyPanel.class.getResource("/bomb_3.png"));
        //System.out.println(MyPanel.class.getResource("/"));

    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);
        g.fillRect(0, 0, 1000, 750); // 填充矩形，默认黑色

        if (hero != null && hero.isLive) {
            // 画出自己坦克--封装方法
            drawTank(hero.getX(), hero.getY(), g, hero.getDirect(), 0);
        }


        // 画出敌人的坦克
        for (int i = 0; i < enemyTanks.size(); i++) {
            // 从集合中，取出坦克
            EnemyTank enemyTank = enemyTanks.get(i);

            // 判断当前坦克是否还存活
            if (enemyTank.isLive) {
                // 绘制坦克
                drawTank(enemyTank.getX(), enemyTank.getY(), g, enemyTank.getDirect(), 1);
                // 画出子弹
                for (int j = 0; j < enemyTank.shots.size(); j++) {
                    // 取出子弹
                    Shot shot = enemyTank.shots.get(j);
                    // 绘制子弹
                    if (shot.isLive) {
                        g.draw3DRect(shot.x + 18, shot.y + 20, 3, 3, false);
                    } else {
                        // 从Vector 移除
                        enemyTank.shots.remove(shot);
                    }
                }
            }

        }

        // 画出hero 自己 发出的子弹
        if (hero.shot != null && hero.shot.isLive) {
            g.draw3DRect(hero.shot.x, hero.shot.y, 3, 3, false);
        }

        // 画出hero 自己 发出的子弹，遍历子弹
        for (int i = 0; i < hero.shots.size(); i++) {
            Shot shot = hero.shots.get(i);
            // 如果子弹不为空 并且 子弹的存活状态为真，绘制子弹
            if (shot != null && shot.isLive) {
                g.draw3DRect(shot.x, shot.y, 3, 3, false);
            } else { // 如果子弹对象 已经无效，就从集合中删除该对象
                hero.shots.remove(shot);
            }
        }

        // 如果bombs 集合中有对象，就画出炸弹
        for (int i = 0; i < bombs.size(); i++) {
            try {
                Thread.sleep(100);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            Bomb bomb = bombs.get(i);
            if (bomb.life > 6) {
                g.drawImage(image1, bomb.x, bomb.y, 60, 60, this);
            } else if (bomb.life > 3) {
                g.drawImage(image2, bomb.x, bomb.y, 60, 60, this);
            } else {
                g.drawImage(image3, bomb.x, bomb.y, 60, 60, this);
            }

            // 减少生命值
            bomb.lifeDown();
            //
            if (bomb.life == 0) {
                bombs.remove(bomb);
            }
        }

    }

    /**
     * @param x      坦克的左上角的X坐标
     * @param y      坦克的左上角的y坐标
     * @param g      画笔
     * @param direct 坦克的方法(上:0下:1左:2右:3)
     * @param type   坦克类型（是自己的坦克，还是敌人的坦克）
     */
    public void drawTank(int x, int y, Graphics g, int direct, int type) {

        // 根据不同类型的坦克，设置不同颜色
        switch (type) {
            case 0:
                g.setColor(Color.red);
                break;
            case 1:
                g.setColor(Color.yellow);
                break;
        }

        // 根据坦克的方向，绘制不同方向的坦克
        switch (direct) {
            // 向上
            case 0:
                g.fill3DRect(x, y, 10, 60, false); // 画出左轮子矩形
                g.fill3DRect(x + 30, y, 10, 60, false); // 画出右轮子矩形
                g.fill3DRect(x + 10, y + 10, 20, 40, false); // 画中间的矩形
                g.drawOval(x + 10, y + 20, 20, 20); // 画中间的圆形
                //g.drawLine(x + 20,y + 30, x + 20, y);  // 画中间的直线给
                g.setColor(Color.black);
                g.fill3DRect(x + 17, y - 20, 5, 40, false);  // 炮管
                break;
            // 向下
            case 1:
                g.fill3DRect(x, y, 10, 60, false); // 画出左轮子矩形
                g.fill3DRect(x + 30, y, 10, 60, false); // 画出右轮子矩形
                g.fill3DRect(x + 10, y + 10, 20, 40, false); // 画中间的矩形
                g.drawOval(x + 10, y + 20, 20, 20); // 画中间的圆形
                //g.drawLine(x + 20,y + 30, x + 20, y);  // 画中间的直线给
                g.setColor(Color.black);
                g.fill3DRect(x + 17, y + 38, 5, 40, false);  // 炮管
                break;
            // 向左
            case 2:
                g.fill3DRect(x, y, 60, 10, false); // 画出左轮子矩形
                g.fill3DRect(x, y + 30, 60, 10, false); // 画出右轮子矩形
                g.fill3DRect(x + 10, y + 10, 40, 20, false); // 画中间的矩形
                g.drawOval(x + 20, y + 10, 20, 20); // 画中间的圆形
                g.setColor(Color.black);
                g.fill3DRect(x - 20, y + 17, 40, 5, false);  // 炮管
                break;
            // 向右
            case 3:
                g.fill3DRect(x, y, 60, 10, false); // 画出左轮子矩形
                g.fill3DRect(x, y + 30, 60, 10, false); // 画出右轮子矩形
                g.fill3DRect(x + 10, y + 10, 40, 20, false); // 画中间的矩形
                g.drawOval(x + 20, y + 10, 20, 20); // 画中间的圆形
                g.setColor(Color.black);
                g.fill3DRect(x + 38, y + 17, 40, 5, false);  // 炮管
                break;
        }

    }


    public void hitEnemyTank() {

        // 遍历我们的子弹
        for (int i = 0; i < hero.shots.size(); i++) {
            Shot shot = hero.shots.get(i);
            // 判断子弹是否击中敌人坦克
            if (shot != null && shot.isLive) { // 当子弹存活时
                // 遍历敌人所有坦克
                for (int j = 0; j < enemyTanks.size(); j++) {
                    EnemyTank enemyTank = enemyTanks.get(j);
                    hitTank(shot, enemyTank);
                }
            }

        }

    }


    // 判断我方的子弹是否击中敌人坦克
    public void hitTank(Shot shot, Tank enemyTank) {
        // 判断shot 击中坦克
        switch (enemyTank.getDirect()) {
            case 0:
                if (shot.x > enemyTank.getX() && shot.x < enemyTank.getX() + 80
                        && shot.y > enemyTank.getY() && shot.y < enemyTank.getY() + 40) {

                    shot.isLive = false;
                    enemyTank.isLive = false;

                    // 当被子弹射中，从Vector 集合中删除当前坦克
                    enemyTanks.remove(enemyTank);

                    //创建Bomb 对象，加入集合
                    Bomb bomb = new Bomb(enemyTank.getX(), enemyTank.getY());
                    bombs.add(bomb);
                }
                break;
            case 1:
                if (shot.x > enemyTank.getX() && shot.x < enemyTank.getX() + 40
                        && shot.y > enemyTank.getY() && shot.y < enemyTank.getY() + 60) {

                    shot.isLive = false;
                    enemyTank.isLive = false;

                    // 当被子弹射中，从Vector 集合中删除当前坦克
                    enemyTanks.remove(enemyTank);

                    //创建Bomb 对象，加入集合
                    Bomb bomb = new Bomb(enemyTank.getX(), enemyTank.getY());
                    bombs.add(bomb);
                }
                break;
            case 2:
                if (shot.x > enemyTank.getX() && shot.x < enemyTank.getX() + 60
                        && shot.y > enemyTank.getY() && shot.y < enemyTank.getY() + 40) {

                    shot.isLive = false;
                    enemyTank.isLive = false;

                    // 当被子弹射中，从Vector 集合中删除当前坦克
                    enemyTanks.remove(enemyTank);

                    //创建Bomb 对象，加入集合
                    Bomb bomb = new Bomb(enemyTank.getX(), enemyTank.getY());
                    bombs.add(bomb);
                }
                break;
            case 3:
                if (shot.x > enemyTank.getX() && shot.x < enemyTank.getX() + 40
                        && shot.y > enemyTank.getY() && shot.y < enemyTank.getY() + 60) {

                    shot.isLive = false;
                    enemyTank.isLive = false;

                    // 当被子弹射中，从Vector 集合中删除当前坦克
                    enemyTanks.remove(enemyTank);

                    //创建Bomb 对象，加入集合
                    Bomb bomb = new Bomb(enemyTank.getX(), enemyTank.getY());
                    bombs.add(bomb);
                }
                break;
        }
    }


    // 判断敌人的坦克是否击中我方坦克
    public void hitHero() {
        //遍历所有的敌人坦克
        for (int i = 0; i < enemyTanks.size(); i++) {
            // 取出一个敌方坦克
            EnemyTank enemyTank = enemyTanks.get(i);
            // 遍历敌人 对象的所有子弹
            for (int j = 0; j < enemyTank.shots.size(); j++) {
                // 取出一个敌方子弹
                Shot shot = enemyTank.shots.get(j);
                // 判断 shot 子弹，是否击中我方坦克
                if (hero.isLive && shot.isLive) {
                    hitTank(shot, hero);
                }
            }
        }
    }

    // 又字符输出时，该方法触发
    @Override
    public void keyTyped(KeyEvent e) {

    }

    // 当某个键被按下时，该方法触发
    // w a s d
    @Override
    public void keyPressed(KeyEvent e) {
        System.out.println(e.getKeyCode());
        // 如果按下的键为下，改变方向 W：上 S:下 A：左 D：右

        if (e.getKeyCode() == KeyEvent.VK_W) {
            hero.setDirect(0);
            // 修改坦克的坐标，移动
            // 如果当前坦克坐标在边界内，才可以向上移动
            if (hero.getY() > 20) {
                hero.moveUp();
            }
        } else if (e.getKeyCode() == KeyEvent.VK_S) {
            hero.setDirect(1);
            // 修改坦克的坐标，移动
            // 如果当前坦克坐标在边界内，才可以向下移动
            if (hero.getY() + 115 < 750) {
                hero.moveDown();
            }

        } else if (e.getKeyCode() == KeyEvent.VK_A) {
            hero.setDirect(2);
            // 修改坦克的坐标，移动
            // 如果当前坦克坐标在边界内，才可以向左移动
            if (hero.getX() > 20) {
                hero.moveLeft();
            }

        } else if (e.getKeyCode() == KeyEvent.VK_D) {
            hero.setDirect(3);
            // 修改坦克的坐标，移动
            // 如果当前坦克坐标在边界内，才可以向右移动
            if (hero.getX() + 95 < 1000) {
                hero.moveRight();
            }

        }

        // 如果按下 J 键，发射子弹
        if (e.getKeyCode() == KeyEvent.VK_J) {

            hero.shotEnemyTank();

        }

        // 面板重绘
        this.repaint();
    }

    // 当某个键被松开时，该方法触发
    @Override
    public void keyReleased(KeyEvent e) {

    }

    @Override
    public void run() { // 每隔 100 毫秒重绘
        while (true) {
            try {
                Thread.sleep(100);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }

            hitEnemyTank();
            hitHero();
            // 重绘
            this.repaint();
        }
    }
}
